## Function that simulates a sample theta of dimension n from a Generalized von Mises distribution ##

GvM <- function(n,m1,m2,k1,k2){
  
  
  ###################################
  
  
  library(pracma)
  
  nn <- 1e5
  
  d <- (m1-m2)%%pi
  sd <- sin(d)
  cd <- cos(d)
  r <- k1/(4*k2)
  coef <- c(1,-4*r*sd*cd,r^2-1,2*r*sd*cd,(sd*cd)^2) # coeffients b0,b1,b2,b3 #
  rts <- roots(coef) # roots of the polynomial q(x) #
  t <- c(asin(rts),(pi-asin(rts)))
  t <- t[!is.nan(t)]
  st <-sin(t)    
  ct <- cos(t)  
  f <- (1-2*sd^2)*(st*ct)-2*sd*cd*(st*st)+r*st+sd*cd      
  af <- abs(f)  
  tsel <- c(t[af<0.01],0)
  cos1 <- cos(tsel) 
  cos2 <- cos(2*(tsel+d)) 
  m <- max(abs(exp(k1*(cos1)+k2*(cos2))))
  
  #####
  
  theta <- rep(0,nn) 
  U <- rand(nn,2)
  U1 <- U[,1]*2*pi
  U2 <- U[,2]*m
  
  #####
  
  index <- (U2<exp(k1*cos(U1-m1)+k2*cos(2*(U1-m2))))
  theta <- U1[index]
  theta <- theta[(1:n)]
  return(theta)
  
}